function eulerToAxis(%euler)
{
	%euler = VectorScale(%euler,$pi / 180);
	%matrix = MatrixCreateFromEuler(%euler);
	return getWords(%matrix,3,6);
}

datablock AudioProfile(bear360Sound)
{
   filename    = "./bear360.wav";
   description = AudioClose3d;
   preload = true;
};

function axisToEuler(%axis)
{
	%angleOver2 = getWord(%axis,3) * 0.5;
	%angleOver2 = -%angleOver2;
	%sinThetaOver2 = mSin(%angleOver2);
	%cosThetaOver2 = mCos(%angleOver2);
	%q0 = %cosThetaOver2;
	%q1 = getWord(%axis,0) * %sinThetaOver2;
	%q2 = getWord(%axis,1) * %sinThetaOver2;
	%q3 = getWord(%axis,2) * %sinThetaOver2;
	%q0q0 = %q0 * %q0;
	%q1q2 = %q1 * %q2;
	%q0q3 = %q0 * %q3;
	%q1q3 = %q1 * %q3;
	%q0q2 = %q0 * %q2;
	%q2q2 = %q2 * %q2;
	%q2q3 = %q2 * %q3;
	%q0q1 = %q0 * %q1;
	%q3q3 = %q3 * %q3;
	%m13 = 2.0 * (%q1q3 - %q0q2);
	%m21 = 2.0 * (%q1q2 - %q0q3);
	%m22 = 2.0 * %q0q0 - 1.0 + 2.0 * %q2q2;
	%m23 = 2.0 * (%q2q3 + %q0q1);
	%m33 = 2.0 * %q0q0 - 1.0 + 2.0 * %q3q3;
	return mRadToDeg(mAsin(%m23)) SPC mRadToDeg(mAtan(-%m13, %m33)) SPC mRadToDeg(mAtan(-%m21, %m22));
}
datablock ParticleData(BearTrailParticle)
{
	dragCoefficient		= 3.0;
	windCoefficient		= 0.0;
	gravityCoefficient	= 0.0;
	inheritedVelFactor	= 0.0;
	constantAcceleration	= 0.0;
	lifetimeMS		= 10000;
	lifetimeVarianceMS	= 0;
	spinSpeed		= 10.0;
	spinRandomMin		= -50.0;
	spinRandomMax		= 50.0;
	useInvAlpha		= false;
	animateTexture		= false;
	//framesPerSec		= 1;

	textureName		= "base/data/particles/dot";
	//animTexName		= "~/data/particles/dot";

	// Interpolation variables
	colors[0]	= "1 1 1 0.2";
	colors[1]	= "1 1 1 0.0";
	sizes[0]	= 0.2;
	sizes[1]	= 0.01;
	times[0]	= 0.0;
	times[1]	= 1.0;
};

datablock ParticleEmitterData(BearTrailEmitter)
{
   ejectionPeriodMS = 2;
   periodVarianceMS = 0;

   ejectionVelocity = 0; //0.25;
   velocityVariance = 0; //0.10;

   ejectionOffset = 0;

   thetaMin         = 0.0;
   thetaMax         = 90.0;  

   particles = arrowTrailParticle;

   useEmitterColors = true;
   uiName = "Bear360 Trail";
};


AddDamageType("Bear360",   '<bitmap:add-ons/Weapon_Bear360Calories/CI_No360> %1',    '%2 <bitmap:add-ons/Weapon_Bear360Calories/CI_No360> %1',0.2,1);
AddDamageType("Bear360SWAG",   '<bitmap:add-ons/Weapon_Bear360Calories/CI_Bear360Calories> %1',    '%2 <bitmap:add-ons/Weapon_Bear360Calories/CI_Bear360Calories> %1',0.2,1);

datablock ProjectileData(Bear360Projectile : gunProjectile)
{
   projectileShapeName = "add-ons/weapon_gun/bullet.dts";
   directDamage        = 40;
   directDamageType    = $DamageType::Bear360;
   radiusDamageType    = $DamageType::Bear360;

   brickExplosionRadius = 0;
   brickExplosionImpact = true;          //destroy a brick if we hit it directly?
   brickExplosionForce  = 10;
   brickExplosionMaxVolume = 1;          //max volume of bricks that we can destroy
   brickExplosionMaxVolumeFloating = 2;  //max volume of bricks that we can destroy if they aren't connected to the ground
	particleEmitter       = BearTrailEmitter;
   muzzleVelocity      = 90;
   velInheritFactor    = 1;

   uiName = "";
};

datablock ItemData(SwagGunItem : GunItem)
{
   uiname = "Bear 360 Calories";
   image = SwagGunImage;
   shapeFile = "./bear360.dts";
   
   doColorShift = true;
   colorShiftColor = "0.8 0.8 0.8 1.000";
};
datablock ShapeBaseImageData(SwagGunImage : GunImage)
{
   item = SwagGunItem;
   shapeFile = "./bear360.dts";
   
	projectile = Bear360Projectile;
	
	doColorShift = true;
   colorShiftColor = "0.8 0.8 0.8 1.000";
   
   stateName[0]                     = "Activate";
	stateTimeoutValue[0]             = 0.3;
	stateEmitterNode[0]				= "trail";
	stateEmitter[0]					= gunSmokeEmitter;
	stateEmitterTime[0]				= 1.50;
	stateTransitionOnTimeout[0]       = "Ready";
	stateSound[0]					= weaponSwitchSound;

	stateName[1]                     = "Ready";
	stateTransitionOnTriggerDown[1]  = "Fire";
	stateAllowImageChange[1]         = true;
	stateSequence[1]	= "Ready";

	stateName[2]                    = "Fire";
	stateTransitionOnTimeout[2]     = "Smoke";
	stateTimeoutValue[2]            = 0.14;
	stateFire[2]                    = true;
	stateAllowImageChange[2]        = false;
	stateSequence[2]                = "Fire";
	stateScript[2]                  = "onFire";
	stateWaitForTimeout[2]			= true;
	stateEmitter[2]					= GunFlashEmitter;
	stateEmitterTime[2]				= 0.05;
	stateEmitterNode[2]				= "muzzleNode";
	stateSound[2]					= Bear360Sound;
	stateEjectShell[2]       = true;

	stateName[3] = "Smoke";
	stateEmitter[3]					= gunSmokeEmitter;
	stateEmitterTime[3]				= 2.00;
	stateEmitterNode[3]				= "muzzlePoint";
	stateTimeoutValue[3]            = 2.00;
	stateTransitionOnTimeout[3]     = "Reload";
	stateWaitForTimeout[3]			= true;

	stateName[4]			= "Reload";
	stateSequence[4]                = "Reload";
	stateTransitionOnTriggerUp[4]     = "Ready";
	//stateSequence[4]	= "Ready";
};
function vectorToAxis( %vec )
{
	%x = getWord( %vec, 0 );
	%y = getWord( %vec, 1 );
	
	%yawAng = mAtan( %x, %y );

	if( %yawAng < 0.0 )
	{
		%yawAng += $pi*2;
	}
	
	%rot = "0 0 1" SPC %yawAng;
	
	return %rot;
}
function SwagGunImage::onMount(%this, %obj, %slot)
{
   Parent::onMount(%this, %obj, %slot);
   if(%obj.lastSwagSpin+1250>getSimTime())
      return;
   %obj.lastSwagSpin=getSimTime();
   %obj.spinStart=getVectorAngle(%obj.getEyeVector());
   %obj.spinStartVel=%obj.getVelocity();
   %obj.spinCurr=0;
   %obj.spinPos=%obj.getPosition();
   %obj.spin();
}
function SwagGunImage::onFire(%this,%obj,%slot)
{
   if(%obj.lastSwagSpin+1250>getSimTime())
   {
      %mask = ($TypeMasks::FxBrickObjectType | $TypeMasks::PlayerObjectType | $TypeMasks::StaticObjectType | $TypeMasks::TerrainObjectType | $TypeMasks::VehicleObjectType);
      %fvec		= %obj.getForwardVector();
		%fX			= getWord(%fvec,0);
		%fY			= getWord(%fvec,1);
		
		%evec		= %obj.getEyeVector();
		%eX			= getWord(%evec,0);
		%eY			= getWord(%evec,1);
		%eZ			= getWord(%evec,2);
		
		%eXY		= mSqrt((%eX * %eX) + (%eY * %eY));
		%aimVec		= (%fX * %eXY) SPC (%fY * %eXY) SPC %eZ;
		
		// Range
		%range		= 1000 * getWord(%obj.getScale(), 2);
		
		if(%range >= 100)
		{
			%rangeRem = %range - 100;
			%range = 100;
		}
		
		// Raycast Parameters
		%start		= %obj.getEyePoint();
		%end[0]		= vectorAdd(%start, vectorScale(%aimVec, %range)); // [0] required for loop
		%targets	= %mask;
		
		// Raycasting
		%ray		= ContainerRayCast(%start, %end[0], %targets, %obj);
		%col		= getWord(%ray, 0);
		
		if(isObject(%col))
		{
			%pos	= posFromRaycast(%ray);
			%normal	= normalFromRaycast(%ray);
			%end[0]	= %pos;
			%colclass	= %col.getClassName();
			%objsHit++;
			
			if(%colClass $= "Player" && !%killed[%col])
         {
            %killed[%col]=1;
         }
         
			%this.onHitObject(%obj, %slot, %col, %pos, %normal, %objsHit);
			
			if(%colclass $= "Player" || %colclass $= "AIPlayer")
			{
				if(%col.getState() $= "Dead")
				{
					%objsKilled++;
				}
			}
		}
		
		if(%rangeRem > 0)
		{
			for(%a = 1; %a <= 10 && %rangeRem > 0; %a++)
			{
				%range		= %rangeRem;
				
				if(%range >= 100)
				{
					%rangeRem = %range - 100;
					%range = 100;
				}
				
				%start[%a]	= %end[%a - 1];
				%end[%a]	= vectorAdd(%start[%a], vectorScale(%aimVec, %range)); 
				
				%ray		= ContainerRayCast(%start[%a], %end[%a], %targets, %obj);
				%col		= getWord(%ray, 0);
				
				if(isObject(%col))
				{
					%pos		= posFromRaycast(%ray);
					%normal		= normalFromRaycast(%ray);
					%end[%a]	= %pos;
					%colclass	= %col.getClassName();
					%objsHit++;
					
					if(%colClass $= "Player" && !%killed[%col])
					{
					   %killed[%col]=1;
					}
					
					%this.onHitObject(%obj, %slot, %col, %pos, %normal, %objsHit);
					
					if(%colclass $= "Player" || %colclass $= "AIPlayer")
					{
						if(%col.getState() $= "Dead")
						{
							%objsKilled++;
						}
					}
					
					if(%objsHit >= 4)
					{
						break;
					}
				}
			}
		}
		if(%objsKilled >= 2)
		   messageAll('',"\c3"@%obj.client.name@"\c6 pulled a \c5"@360*%objsKilled@" No-Scope\c6!");
   }
   else
   {
      Parent::onFire(%this,%obj,%slot);
   }
}
function SwagGunImage::onHitObject(%this, %obj, %slot, %col, %pos, %normal, %objsHit)
{ 
	if(%col.getType() & ($TypeMasks::PlayerObjectType | $TypeMasks::VehicleObjectType))
	{
		if(miniGameCanDamage(%obj, %col) == 1 && !%col.getDatablock().indestructable)
		{
			%damageType    = $DamageType::Bear360SWAG;
			%scale			= %obj.getScale();
			%scaleFactor	= (getWord(%scale, 0) + getWord(%scale, 1) + getWord(%scale, 2)) / 3;
			%damFloat		= mClampF(%this.raycastDirectDamage, -100, 100);
			%directDamage	= 1000;
			
			%name=%col.client.name;
			%col.damage(%obj, %pos, %directDamage, %damageType);
			
			
         %obj.emote("winStarProjectile;",1);
         serverplay3d(rewardSound,%obj.getTransform());
         messageAll('',"\c3"@%obj.client.name@"\c6 360 No-Scoped \c3"@%name);
		}
	}
	
   %scale			= %obj.getScale();
   %scaleFactor	= ((getWord(%scale, 0) + getWord(%scale, 1) + getWord(%scale, 2)) / 3) / %objsHit;
   
   %p = new Projectile()
   {
      dataBlock		= Bear360Projectile;
      initialPosition	= %pos;
      initialVelocity	= %normal;
      sourceObject	= %obj;
      client			= %obj.client;
      sourceSlot		= 0;
      originPoint		= %pos;
   };
   
   MissionCleanup.add(%p);
   %p.setScale(%scaleFactor SPC %scaleFactor SPC %scaleFactor);
   %p.explode();
	
   //serverplay3d(AudioProfile, %pos);
}
function Player::Spin(%obj)
{
   cancel(%obj.spinSched);
   %obj.setVelocity("0 0 0");
   %obj.spinCurr+=10;
   if(%obj.spinCurr>270)
   {
      %obj.setVelocity(%obj.spinStartVel);
      return;
   }
   %obj.setTransform(%obj.spinPos SPC eulerToAxis("0 0 "@niceEuler(%obj.spinStart + %obj.spinCurr)));
   %obj.spinSched=%obj.schedule(16,spin);
}
function getVectorAngle(%v)
{
   return mRadToDeg(mATan(getWord(%v,1),getWord(%v,0)));
}
function niceEuler(%a)
{
   %a=%a % 360;
   if(%a>180)
      %a=-(360-%a);
   return %a;
}
